<?php
/**
 * @package     3.x
 * @subpackage  J2 Store Easy Checkout
 * @author      Alagesan, J2Store <support@j2store.org>
 * @copyright   Copyright (c) 2018 J2Store . All rights reserved.
 * @license     GNU GPL v3 or later
 * @link        http://j2store.org
 * --------------------------------------------------------------------------------
 *
 * */
// No direct access to this file
defined( '_JEXEC' ) or die;

class EasyCheckoutControllerCheckouts extends F0FController
{

	protected $cacheableTasks = array();

	function execute( $task )
	{
		if ( in_array( $task, array( 'add', 'edit', 'read', 'browse' ) ) )
		{
			$task = 'browse';
		}

		return parent::execute( $task );
	}

	protected function onBeforeGenericTask( $task )
	{
		$format    = JFactory::getApplication()->input->getString( 'format', '' );
		$forbidden = array( 'json', 'csv', 'pdf' );
		if ( in_array( strtolower( $format ), $forbidden ) )
		{
			return false;
		}

		return parent::onBeforeGenericTask( $task );
	}

	protected function onBeforeBrowse()
	{

		$format    = JFactory::getApplication()->input->getString( 'format', '' );
		$forbidden = array( 'json', 'csv', 'pdf' );
		if ( in_array( strtolower( $format ), $forbidden ) )
		{
			return false;
		}

		return parent::onBeforeBrowse();
	}

	public function display( $cachable = false, $urlparams = array(), $tpl = null )
	{

		$document = F0FPlatform::getInstance()->getDocument();
		$app      = JFactory::getApplication();
		$user     = JFactory::getUser();

		if ( $document instanceof JDocument )
		{
			$viewType = $document->getType();
		} else
		{
			$viewType = $this->input->getCmd( 'format', 'html' );
		}

		$view = $this->getThisView();

		// Get/Create the model

		if ( $model = $this->getThisModel() )
		{
			// Push the model into the view (as default)
			$view->setModel( $model, true );
		}

		$isLogged = 0;
		if ( $user->id )
		{
			$isLogged = 1;
		}
		$view->assign( 'logged', $isLogged );

		$order     = F0FModel::getTmpInstance( 'Orders', 'J2StoreModel' )->initOrder()->getOrder();
		$items     = $order->getItems();
		$session   = JFactory::getSession();
        $session->set ( 'force_calculate_shipping', 1, 'j2store' );
        $shipping_address_id = $session->get('shipping_address_id',0,'j2store');
        $billing_address_id = $session->get('billing_address_id',0,'j2store');
        $user = JFactory::getUser();
        if($user->id && (empty($billing_address_id) || empty($shipping_address_id))){
            // Default Addresses
            $address_table = F0FModel::getTmpInstance( 'Addresses', 'J2StoreModel' )->user_id( $user->id )->getFirstItem();
            $session->set ( 'shipping_country_id', $address_table->country_id,'j2store');
            $session->set ( 'shipping_zone_id', $address_table->zone_id,'j2store');
            $session->set ( 'shipping_postcode', $address_table->zip,'j2store');

            // billing session
            $session->set ( 'billing_country_id', $address_table->country_id,'j2store');
            $session->set ( 'billing_zone_id', $address_table->zone_id,'j2store');
            $session->set ( 'billing_postcode', $address_table->zip,'j2store');
        }
        if($shipping_address_id){
            $address_table = F0FTable::getAnInstance('Address', 'J2StoreTable')->getClone();
            $address_table->load($shipping_address_id);
            // shipping session
            $session->set ( 'shipping_country_id', $address_table->country_id,'j2store');
            $session->set ( 'shipping_zone_id', $address_table->zone_id,'j2store');
            $session->set ( 'shipping_postcode', $address_table->zip,'j2store');
        }
        if($billing_address_id){
            $bill_address_table = F0FTable::getAnInstance('Address', 'J2StoreTable')->getClone();
            $bill_address_table->load($shipping_address_id);
            // billing session
            $session->set ( 'billing_country_id', $bill_address_table->country_id,'j2store');
            $session->set ( 'billing_zone_id', $bill_address_table->zone_id,'j2store');
            $session->set ( 'billing_postcode', $bill_address_table->zip,'j2store');
        }

		$is_mobile = $session->get( 'is_mobile', '', 'j2store' );

		$mobile = ( $is_mobile ) ? '&mobile=mobile' : '';
		$link   = JRoute::_( 'index.php?option=com_j2store&view=carts' . $mobile );

        $params = JComponentHelper::getParams('com_easycheckout');
        $enable_easycheckout = $params->get('enable_easycheckout',1);
        if(!$enable_easycheckout){
            $link   = JRoute::_( 'index.php?option=com_j2store&view=checkouts' . $mobile );
            $app->redirect( $link );
        }

		if ( count( $items ) < 1 )
		{
			$app->enqueueMessage( JText::_( 'J2STORE_CART_NO_ITEMS' ), 'notice' );
			$app->redirect( $link );

		}

		//validate stock
		if ( $order->validate_order_stock() == false )
		{
			$app->redirect( $link );
		}
		//prepare shipping
		// Checking whether shipping is required
		$showShipping = false;

		if ( J2Store::config()->get( 'show_shipping_address', 0 ) )
		{
			$showShipping = true;
		}

		if ( $isShippingEnabled = $order->isShippingEnabled() )
		{
			$showShipping = true;
		}
        $view->assign( 'showShipping', $showShipping );
		//prepare the billing address object
		$this->prepare_billing_address( $view, $order, $user );


		// there could be three scenarios

		//1: Customer is already logged in
		if ( $user->id > 0 )
		{
			$this->customer_logged_in( $view, $order, $user );
		}else {
			$this->guest_checkout( $view, $order, $user );
		}

		if($showShipping){
            $shipping_layout = "shipping_yes";
            $shipping_method_form = $this->getShippingHtml( $shipping_layout, $order );
            $view->assign( 'showShipping', $showShipping );
            $view->assign( 'shipping_method_form', $shipping_method_form );
        }
        //custom fields
        $selectableBase = J2Store::getSelectableBase();
        $address_table = F0FTable::getAnInstance('Address', 'J2StoreTable');
        $payment_fields = $selectableBase->getFields('payment',$address_table,'address');

        $view->assign( 'payment_fields', $payment_fields );

        //process payment plugins
        $showPayment = true;
        if ((float)$order->order_total == (float)'0.00'  )
        {
            if(isset($order->show_payment_method) && $order->show_payment_method == 1){
                $showPayment = true;
            }else{
                $showPayment = false;
            }
            $app->triggerEvent("onJ2StoreChangeShowPaymentOnTotalZero", array( $order, &$showPayment ) );
        }
        $view->assign( 'showPayment', $showPayment );

        $payment_plugins = J2Store::plugin()->getPluginsWithEvent( 'onJ2StoreGetPaymentPlugins' );
        $default_method = J2Store::config()->get('default_payment_method', '');
        $plugins = array();
        if ($payment_plugins)
        {
            foreach ($payment_plugins as $plugin)
            {
                $results = $app->triggerEvent("onJ2StoreGetPaymentOptions", array( $plugin->element, $order ) );
                if (!in_array(false, $results, false))
                {
                    if(!empty($default_method) && $default_method == $plugin->element) {
                        $plugin->checked = true;
                        $html = $this->getPaymentForm( $plugin->element, true);
                        $view->assign( 'payment_form_div', $html);
                    }
                    $plugins[] = $plugin;
                }
            }
        }

        if (count($plugins) == 1)
        {
            $plugins[0]->checked = true;
            $html = $this->getPaymentForm( $plugins[0]->element, true);
            $view->assign( 'payment_form_div', $html);
        }

        $view->assign('plugins', $plugins);


		$view->assign( 'order', $order );
        // Set the layout
        $view->setLayout( is_null( $this->layout ) ? 'default' : $this->layout );
		//trigger on before checkout event
		J2Store::plugin()->event( 'BeforeCheckout', array( $order, &$view ) );
		// Display without caching
		$view->display();
	}

	public function guest_checkout($view, $order, $user){

    }

	/**
     * get Shipping Plugin list
     * @param $layout - layout of shipping html
     * @param $order -  order object
     * @return string
	*/
    function getShippingHtml($layout = 'shipping_yes', &$order) {
        $html = '';
        $view = $this->getThisView ();
        if ($model = $this->getThisModel ()) {
            // Push the model into the view (as default)
            $view->setModel ( $model, true );
        }

        $view->setLayout ( $layout );

        $rates = array ();

        switch (strtolower ( $layout )) {
            case "shipping_calculate" :
                break;
            case "shipping_no" :
                break;
            case "shipping_yes" :
            default :
                $rates = F0FModel::getTmpInstance ( 'Shippings', 'J2StoreModel' )->getShippingRates ( $order );
                $default_rate = array ();
                $session = JFactory::getSession ();
                $shipping_values = $session->get ( 'shipping_values', array (), 'j2store' );
                if(count($rates)){
                    $order->show_payment_method = 1;
                }else{
                    $order->show_payment_method = 0;
                }
                if (count ( $rates ) == 1) {
                    $default_rate = $rates [0];
                } elseif (count ( $shipping_values )) {
                    foreach ( $rates as $rate ) {
                        if ($rate ['name'] == $shipping_values ['shipping_name']) {
                            $shipping_values ['name'] = $shipping_values ['shipping_name'];
                            $shipping_values ['price'] = $shipping_values ['shipping_price'];
                            $shipping_values ['code'] = $shipping_values ['shipping_code'];
                            $shipping_values ['tax'] = $shipping_values ['shipping_tax'];
                            $shipping_values ['extra'] = $shipping_values ['shipping_extra'];
                            $shipping_values ['element'] = $rate ['element'];
                            $default_rate = $shipping_values;
                        }
                    }
                }

                $view->assign ( 'rates', $rates );
                $view->assign ( 'default_rate', $default_rate );
                break;
        }

        ob_start ();
        $view->display ();
        $html = ob_get_contents ();
        ob_end_clean ();
        return $html;
    }

    /**
     * Get Payment Method list
     * @param $element - default element
     * @param $plain_format - return format type
     * @param mixing
    */
    function getPaymentForm($element = '', $plain_format = false) {
        $app = JFactory::getApplication ();
        $values = $app->input->getArray ( $_REQUEST );
        $html = '';
        $text = "";
        $user = JFactory::getUser ();
        if (empty ( $element )) {
            $element = $app->input->getString ( 'payment_element' );
        }
        $results = array ();

        JPluginHelper::importPlugin ( 'j2store' );

        $results = $app->triggerEvent ( "onJ2StoreGetPaymentForm", array (
            $element,
            $values
        ) );
        for($i = 0; $i < count ( $results ); $i ++) {
            $result = $results [$i];
            $text .= $result;
        }

        $html = $text;
        if ($plain_format) {
            return $html;
        } else {

            // set response array
            $response = array ();
            $response ['msg'] = $html;

            // encode and echo (need to echo to send back to browser)
            echo json_encode ( $response );
            $app->close ();
        }
    }

    /**
     * login validation
     *
    */
	function login_validate()
	{

		$app     = JFactory::getApplication();
		$user    = JFactory::getUser();
		$session = JFactory::getSession();
		$params  = J2Store::config();
		$session->set( 'uaccount', 'login', 'j2store' );

		$view = $this->getThisView();

		if ( $model = $this->getThisModel() )
		{
			// Push the model into the view (as default)
			$view->setModel( $model, true );
		}

		$redirect_url = JRoute::_( 'index.php?option=com_easycheckout&view=checkout' );

		$json = array();

		if ( $user->id )
		{
			$json['redirect'] = $redirect_url;
		}
		J2Store::plugin()->eventWithArray( 'CheckoutBeforeLogin', array( &$json ) );

		if ( ! $json )
		{

			$userHelper = J2Store::user();
			//now login the user
			if ( ! $userHelper->login(
				array(
					'username' => $app->input->getString( 'email' ),
					'password' => $app->input->getRaw( 'password' )
				)
			)
			)
			{
				$json['error']['warning'] = JText::_( 'J2STORE_CHECKOUT_ERROR_LOGIN' );
			}

		}

		if ( ! $json )
		{
			//clear if there are any guest values
			$session->clear( 'guest', 'j2store' );
			$user = JFactory::getUser();
			// Default Addresses
			$address_info = F0FModel::getTmpInstance( 'Addresses', 'J2StoreModel' )->user_id( $user->id )->getFirstItem();

			if ( $address_info )
			{
				if ( $params->get( 'config_tax_default' ) == 'shipping' )
				{
					$session->set( 'shipping_country_id', $address_info->country_id, 'j2store' );
					$session->set( 'shipping_zone_id', $address_info->zone_id, 'j2store' );
					$session->set( 'shipping_postcode', $address_info->zip, 'j2store' );
				}

				if ( $params->get( 'config_tax_default' ) == 'billing' )
				{
					$session->set( 'billing_country_id', $address_info->country_id, 'j2store' );
					$session->set( 'billing_zone_id', $address_info->zone_id, 'j2store' );
					$session->set( 'billing_postcode', $address_info->zip, 'j2store' );
				}
			} else
			{
				$session->clear( 'shipping_country_id', 'j2store' );
				$session->clear( 'shipping_zone_id', 'j2store' );
				$session->clear( 'shipping_postcode', 'j2store' );
				$session->clear( 'billing_country_id', 'j2store' );
				$session->clear( 'billing_zone_id', 'j2store' );
				$session->clear( 'billing_postcode', 'j2store' );
			}

			$json['redirect'] = $redirect_url;
		}

		J2Store::plugin()->eventWithArray( 'CheckoutAfterLogin', array( &$json ) );

		echo json_encode( $json );
		$app->close();
	}

	/**
     * get Billing Address needed data
     * @param $view - view object
     * @param $order - order object
     * @param $user - Joomla user object
     *
	*/
	public function prepare_billing_address(&$view, &$order, $user) {
		$selectableBase = J2Store::getSelectableBase();
		$view->assign('fieldsClass', $selectableBase);

		$address = F0FTable::getAnInstance('address', 'J2StoreTable')->getClone();
		$fields = $selectableBase->getFields('billing',$address,'address');
		J2Store::plugin ()->event ( 'BeforeCheckoutRegister', array(&$address,$order) );

		//if the user is logged in, we can attempt to retrieve the user profile data.

		if($user->id) {
			//$address = $address_model->user_id($user->id)->getFirstItem();
			if(isset( $address->j2store_address_id ) && empty($address->j2store_address_id)){
				$userProfile = JUserHelper::getProfile( $user->id );
				$address->address_1 = isset($userProfile->profile['address1']) ? $userProfile->profile['address1']:'';
				$address->address_2 = isset($userProfile->profile['address2']) ? $userProfile->profile['address2']:'';
				$address->city = isset($userProfile->profile['city']) ? $userProfile->profile['city']:'';
				$address->zip = isset($userProfile->profile['postal_code']) ? $userProfile->profile['postal_code']:'';
				//
				$address->phone_1 = isset($userProfile->profile['phone']) ? $userProfile->profile['phone']:'';
				$address->phone_2 = isset($userProfile->profile['mobilephone']) ? $userProfile->profile['mobilephone']:'';
				$address->first_name = isset($userProfile->profile['first_name']) ? $userProfile->profile['first_name']:'';
				$address->last_name = isset($userProfile->profile['last_name']) ? $userProfile->profile['last_name']:'';
			}

		}


		// if any custom field, need to add address object
		$default_address = F0FTable::getAnInstance('Address', 'J2StoreTable')->getClone();
		$selectableBase->getFields('billing',$default_address,'address');

		$view->assign('fields', $fields);
		$view->assign('address', empty($addresses) ? $address :$default_address);

	}


	/**
     * Set Customer details in view object
     * @param $view - view object
     * @param $order - order object
     * @param $user - Joomla User object
	*/
	public function customer_logged_in(&$view, &$order, $user) {

		$session = JFactory::getSession();

		//load all his adddresses
		$addresses = $this->getAllAddresses();
		$view->assign('addresses', $addresses);

		//probably he might not be have any address stored.
		if(count($addresses) < 1) {
			$view->assign('noaddress_stored', 1);
		}

		$address = $view->address;
		//get the billing address id from the session. Possibility that the customer might have selected one already
		if ($session->has('billing_address_id', 'j2store')) {
			$billing_address_id = $session->get('billing_address_id', '', 'j2store');
		} else {
			$billing_address_id = isset($address->j2store_address_id)?$address->j2store_address_id:'';
		}
		$view->assign('address_id', $billing_address_id);

		//check if the customer has already selected a country and zone
		if ($session->has('billing_country_id', 'j2store')) {
			$billing_country_id = $session->get('billing_country_id', '', 'j2store');
		} else {
			$billing_country_id = isset($address->country_id)?$address->country_id:'';
		}
		$view->assign('country_id', $billing_country_id);

		if ($session->has('billing_zone_id', 'j2store')) {
			$billing_zone_id = $session->get('billing_zone_id', '', 'j2store');
		} else {
			$billing_zone_id = isset($address->zone_id)?$address->zone_id:'';
		}
		$view->assign('zone_id', $billing_zone_id);
	}

    public function get_checkout_form(){
        $app = JFactory::getApplication();
        $address_type = $app->input->get('address_type','billing');
        $session = JFactory::getSession();
        $address_model = F0FModel::getTmpInstance('Addresses', 'J2StoreModel');

        $view = $this->getThisView ();
        if ($model = $this->getThisModel ()) {
            // Push the model into the view (as default)
            $view->setModel ( $model, true );
        }
        $user = JFactory::getUser();
        $address = F0FTable::getAnInstance('Address', 'J2StoreTable');
        if($user->id) {
            //$address = $address_model->user_id($user->id)->getFirstItem();
            if(isset( $address->j2store_address_id ) && empty($address->j2store_address_id)){
                $userProfile = JUserHelper::getProfile( $user->id );
                $address->address_1 = isset($userProfile->profile['address1']) ? $userProfile->profile['address1']:'';
                $address->address_2 = isset($userProfile->profile['address2']) ? $userProfile->profile['address2']:'';
                $address->city = isset($userProfile->profile['city']) ? $userProfile->profile['city']:'';
                $address->zip = isset($userProfile->profile['postal_code']) ? $userProfile->profile['postal_code']:'';
                //
                $address->phone_1 = isset($userProfile->profile['phone']) ? $userProfile->profile['phone']:'';
                $address->phone_2 = isset($userProfile->profile['mobilephone']) ? $userProfile->profile['mobilephone']:'';
                $address->first_name = isset($userProfile->profile['first_name']) ? $userProfile->profile['first_name']:'';
                $address->last_name = isset($userProfile->profile['last_name']) ? $userProfile->profile['last_name']:'';
            }

        }

        $is_mobile = $session->get('is_mobile','','j2store');

        $mobile = ($is_mobile) ? '&mobile=mobile' : '';
        $link = JRoute::_('index.php?option=com_j2store&view=carts'.$mobile);

        $order = F0FModel::getTmpInstance('Orders', 'J2StoreModel')->initOrder()->getOrder();
        if(count($order->getItems()) < 1 ) {
            $app->redirect($link, $order->getError());
        }

        //validate stock
        if($order->validate_order_stock() == false) {
            $app->redirect($link);
        }
        $addresses = array();
        J2Store::plugin ()->event ( 'BeforeCheckoutBilling', array(&$address,$addresses,$order) );

        $selectableBase = J2Store::getSelectableBase();
        $view->assign('fieldsClass', $selectableBase);
        $fields = $selectableBase->getFields($address_type,$address,'address');
        // if any custom field, need to add address object
        $default_address = F0FTable::getAnInstance('Address', 'J2StoreTable')->getClone();
        $selectableBase->getFields($address_type,$default_address,'address');
        $view->assign('address_type', $address_type);
        $view->assign('fields', $fields);
        $view->assign('address', empty($addresses) ? $address :$default_address);
        //get layout settings
        $view->assign('storeProfile', J2Store::storeProfile());
        $view->setLayout ( 'new_address' );
        $html = '';

        ob_start();
        $view->display();
        $html .= ob_get_contents();
        ob_end_clean();
        echo $html;
        $app->close();
    }

    public function validate_checkout(){
	    $app = JFactory::getApplication();
	    $data = $app->input->getArray($_REQUEST);
        $user = JFactory::getUser();
        $session = JFactory::getSession();
        $address_model = F0FModel::getTmpInstance('Addresses', 'J2StoreModel');
        $redirect_url = JRoute::_('index.php?option=com_j2store&view=checkout');
        $selectableBase = J2Store::getSelectableBase();
        $params = J2Store::config();
        if(isset($data['billing']) && !empty($data['billing'])){
            $billing_address_data = $data['billing'];
            unset($data['billing']);
            foreach ($billing_address_data as $b_data){
                $data['billing'][$b_data['name']] = $b_data['value'];
            }
        }

        if(isset($data['shipping']) && !empty($data['shipping'])){
            $shipping_address_data = $data['shipping'];
            unset($data['shipping']);
            foreach ($shipping_address_data as $s_data){
                $data['shipping'][substr($s_data['name'],9)] = $s_data['value'];
            }
        }
        if(isset($data['payment']) && !empty($data['payment'])){
            $payment_address_data = $data['payment'];
            unset($data['payment']);
            foreach ($payment_address_data as $s_data){
                $data['payment'][substr($s_data['name'],8)] = $s_data['value'];
            }
        }

        if(isset($data['shipping_method']) && is_array($data['shipping_method']) && !empty($data['shipping_method'])){
            $shipping_method_datas = $data['shipping_method'];
            unset($data['shipping_method']);
            foreach ($shipping_method_datas as $shipping_method_data){
                $data['shipping_method'][$shipping_method_data['name']] = $shipping_method_data['value'];
            }
        }

        if(isset($data['payment_form_data']) && is_array($data['payment_form_data']) && !empty($data['payment_form_data'])){
            $payment_form_datas = $data['payment_form_data'];
            unset($data['payment_form_data']);
            foreach ($payment_form_datas as $payment_form_data){
                $data['payment_form_data'][$payment_form_data['name']] = $payment_form_data['value'];
            }
        }

        if(isset($data['new_billing_address']) && is_array($data['new_billing_address']) && !empty($data['new_billing_address'])){
            $new_billing_address = $data['new_billing_address'];
            unset($data['new_billing_address']);
            foreach ($new_billing_address as $new_billing_addr){
                $data['new_billing_address'][$new_billing_addr['name']] = $new_billing_addr['value'];
            }
        }

        if(isset($data['new_shipping_address']) && !empty($data['new_shipping_address'])){
            $new_shipping_address_data = $data['new_shipping_address'];
            unset($data['new_shipping_address']);
            foreach ($new_shipping_address_data as $s_data){
                $data['new_shipping_address'][substr($s_data['name'],9)] = $s_data['value'];
            }
        }
        if(isset($data['payment_shipping_input']) && !empty($data['payment_shipping_input'])){
            parse_str(urldecode($data['payment_shipping_input']),$output);
            $data['payment_shipping_input'] = $output;
        }


        $json = array();
        // user billing validation for guest,register
        if($user->id > 0){
            $bill_chk = false;
            $ship_chk = false;
            // if register user enter address
            if(empty($data['new_billing_address']) && (!isset($data['billing_address_type']) || $data['billing_address_type'] != 'existing' )){
                $data['new_billing_address'] = $data['billing'];
                $data['billing_address_type'] = 'new';
                $bill_chk = true;
            }

            if(empty($data['new_shipping_address']) && (!isset($data['billing_address_type']) || $data['billing_address_type'] != 'existing' )){
                $ship_chk = true;
                $data['shipping_address_type'] = 'new';
                if(isset($data['shipping_address']) && $data['shipping_address']){
                    $data['new_shipping_address'] = $data['billing'];
                }else{
                    $data['new_shipping_address'] = $data['shipping'];
                }
            }

            //Login user so need to check old address or new address
            if($bill_chk){
                $this->validate_billing_address($data,$json);
            }
            if($ship_chk){
                $this->validate_shipping_address($data,$json);
            }


            if($bill_chk && isset($json['error']['new_billing_address'])){
                $json['error']['billing'] = $json['error']['new_billing_address'];
            }
            if($ship_chk && isset($json['error']['new_shipping_address'])){
                $json['error']['shipping'] = $json['error']['new_shipping_address'];
            }

        }elseif(isset($data['billing']['create_account']) && $data['billing']['create_account'] == 1){
            $this->validate_register_billing_address($data,$json);
        }else{
            $this->validate_guest_address($data,$json);
        }

        // shipping method and payment method validate
        if(!$json){
            $profile_order_id = $session->get('profile_order_id',null,'j2store');
            $order = F0FModel::getTmpInstance('Orders', 'J2StoreModel')->initOrder($profile_order_id)->getOrder();
            $payment_field = isset($data['payment']) ? $data['payment']: array();
            
            $json = $selectableBase->validate($payment_field, 'payment', 'address');

            if($json){
                $billing_field_error = $json['error'];
                unset($json['error']);
                foreach ( $billing_field_error as $json_key => $json_value){
                    $json['error']['payment'][$json_key] = $json_value;
                }
            }

            if (!$json) {
                //validate weather the customer is logged in
                /*$billing_address = '';
                if ($user->id && $session->has('billing_address_id', 'j2store')) {
                    $billing_address = $address_model->getItem($session->get('billing_address_id', '', 'j2store'));
                } elseif ($session->has('guest', 'j2store')) {
                    $guest = $session->get('guest', array(), 'j2store');
                    $billing_address = isset($guest['billing']) ? $guest['billing']: array();
                }

                if (empty($billing_address)) {
                    $json['redirect'] = $redirect_url;
                }*/

                //cart has products?
                if ($order->getItemCount() < 1) {
                    $json['redirect'] = $redirect_url;
                }

                if (!$json) {
                    $isShippingEnabled = $order->isShippingEnabled();

                    //validate selection of shipping methods and set the shipping rates
                    if($params->get('show_shipping_address', 0) || $isShippingEnabled ) {
                        //shipping is required.
                        /*if ($user->id && $session->has('shipping_address_id', 'j2store')) {
                            $shipping_address = $address_model->getItem($session->get('shipping_address_id', '', 'j2store'));
                        } elseif ($session->has('guest', 'j2store')) {
                            $guest = $session->get('guest', array(), 'j2store');
                            $shipping_address = isset($guest['shipping']) ? $guest['shipping']:array();
                        }

                        //check if shipping address id is set in session. If not, redirect
                        if(empty($shipping_address)) {
                            $json['error']['shipping_method'] = JText::_('J2STORE_CHECKOUT_ERROR_SHIPPING_ADDRESS_NOT_FOUND');
                            //$json['redirect'] = $redirect_url;
                        }*/

                        try {
                            
                            $this->validateSelectShipping($data, $order);
                        } catch (Exception $e) {
                            $json['error']['shipping_method'] = $e->getMessage();
                        }

                        if(!$json) {


                            $shipping_values = array();
                            $shipping_values['shipping_price']    = isset($data['shipping_method']['shipping_price']) ? $data['shipping_method']['shipping_price'] : 0;
                            $shipping_values['shipping_extra']   = isset($data['shipping_method']['shipping_extra']) ? $data['shipping_method']['shipping_extra'] : 0;
                            $shipping_values['shipping_code']     = isset($data['shipping_method']['shipping_code']) ? $data['shipping_method']['shipping_code'] : '';
                            $shipping_values['shipping_name']     = isset($data['shipping_method']['shipping_name']) ? $data['shipping_method']['shipping_name'] : '';
                            $shipping_values['shipping_tax']      = isset($data['shipping_method']['shipping_tax']) ? $data['shipping_method']['shipping_tax'] : 0;
                            $shipping_values['shipping_plugin']     = isset($data['shipping_method']['shipping_plugin']) ? $data['shipping_method']['shipping_plugin'] : '';
                            //set the shipping method to session
                            $session->set('shipping_method',$shipping_values['shipping_plugin'], 'j2store');
                            $session->set('shipping_values',$shipping_values, 'j2store');

                        }
                    }
                }


                if (!$json) {
                    // is shipping mandatory
                    if($params->get('shipping_mandatory', 0)) {
                        //yes it is. Check if session has shipping values
                        $shipping_values = $session->get('shipping_values', array(), 'j2store');
                        $shipping_method = $session->get('shipping_method', null, 'j2store');
                        if(count($shipping_values) < 1 || empty($shipping_method)) {
                            //now value selected
                            $json['error']['shipping_method'] = JText::_('J2STORE_CHECKOUT_SHIPPING_METHOD_SELECTION_MANDATORY');
                        }
                    }
                }

                //validate selection of payment methods
                if (!$json) {
                    $profile_order_id = $session->get('profile_order_id',null,'j2store');
                    //re initialise the order
                    $order = F0FModel::getTmpInstance('Orders', 'J2StoreModel')->initOrder($profile_order_id)->getOrder();

                    $showPayment = true;
                    if ((float)$order->order_total == (float)'0.00')
                    {
                        $showPayment = false;
                        $app->triggerEvent("onJ2StoreChangeShowPaymentOnTotalZero", array( $order, &$showPayment ) );
                    }
                    $form_payment_data = isset($data['payment_form_data']) ? $data['payment_form_data']: array();
                    if($showPayment) {
                        $payment_plugin = $app->input->getString('payment_plugin');
                        if (!isset($payment_plugin)) {
                            $json['error']['payment_method'] = JText::_('J2STORE_CHECKOUT_ERROR_PAYMENT_METHOD');
                        } elseif (!isset($payment_plugin )) {
                            $json['error']['payment_method'] = JText::_('J2STORE_CHECKOUT_ERROR_PAYMENT_METHOD');
                        }
                        //validate the selected payment
                        try {

                            $this->validateSelectPayment($payment_plugin, $form_payment_data);
                        } catch (Exception $e) {
                            $json['error']['payment_method'] = $e->getMessage();
                        }

                    }

                    if($params->get('show_terms', 0) && $params->get('terms_display_type', 'link') =='checkbox' ) {
                        $tos_check = $app->input->get('checkbox_tos');
                        if (isset($tos_check) && $tos_check == 0) {
                            $json['error']['tos_check'] = JText::_('J2STORE_CHECKOUT_ERROR_AGREE_TERMS');
                        }
                    }

                    if (!$json) {
                        if(isset($data['payment']) && !empty($data['payment'])){
                            foreach ($data['payment'] as $payment_key => $payment_value){
                                $form_payment_data[$payment_key] = $payment_value;
                            }
                        }

                        $payment_plugin = $app->input->getString('payment_plugin');
                        //set the payment plugin form values in the session as well.
                        $session->set('payment_values', $form_payment_data, 'j2store');
                        $session->set('payment_method', $payment_plugin, 'j2store');
                        $session->set('customer_note', strip_tags($app->input->getString('customer_note')), 'j2store');
                    }
                }
            }

            //if(!$json) {
                $plugin_data = isset($data['payment_shipping_input']) ? $data['payment_shipping_input'] : $data;

                J2Store::plugin()->eventWithArray('CheckoutValidateShippingPayment',array($plugin_data, $order,&$json));
            //}

        }

        if(!$json){
            $process_type = $app->input->get('process_type','validate');
            if($process_type == 'submit'){
                $this->saveBillingAndShipping($data,$json);
                if(!$json){
                    $json['success'] = true;
                    $json['redirect'] = JRoute::_('index.php?option=com_easycheckout&view=checkout&task=confirm');
                }
            }

        }

        $json['html'] = $this->processHtml($data,$json);
        echo json_encode($json);
        $app->close();
    }

    public function processHtml($data,&$json){
        $app = JFactory::getApplication();
	    $session = JFactory::getSession();

        $country_id = isset($data['billing']['country_id']) ? $data['billing']['country_id']: $session->get('billing_country_id','','j2store');
        $zone_id = isset($data['billing']['zone_id']) ? $data['billing']['zone_id']: $session->get('billing_zone_id','','j2store');
        $postcode = isset($data['billing']['zip']) ? $data['billing']['zip']: $session->get('billing_postcode','','j2store');

        $ship_country_id = isset($data['shipping']['country_id']) ? $data['shipping']['country_id']: $session->get('shipping_country_id','','j2store');
        $ship_zone_id = isset($data['shipping']['zone_id']) ? $data['shipping']['zone_id']: $session->get('shipping_zone_id','','j2store');
        $ship_postcode = isset($data['shipping']['zip']) ? $data['shipping']['zip']: $session->get('shipping_postcode','','j2store');

        if(!$session->has('billing_country_id')){
            $session->set('billing_country_id', $country_id, 'j2store');
        }
        if(!$session->has('billing_zone_id')){
            $session->set('billing_zone_id', $zone_id, 'j2store');
        }
        if(!$session->has('billing_postcode')){
            $session->set('billing_postcode', $postcode, 'j2store');
        }

        $shipping_address = isset($data['shipping_address']) ? $data['shipping_address']: 0;
        if ($shipping_address == 1) {
            $ship_country_id = $country_id;
            $ship_zone_id = $zone_id;
            $ship_postcode = $postcode;

        }
        if(!$session->has('shipping_address_id')){
            $session->set('shipping_country_id', $ship_country_id, 'j2store');
        }
        if(!$session->has('shipping_zone_id')){
            $session->set('shipping_zone_id', $ship_zone_id, 'j2store');
        }
        if(!$session->has('shipping_postcode')){
            $session->set('shipping_postcode', $ship_postcode, 'j2store');
        }
        $view = $this->getThisView();

        // Get/Create the model

        if ( $model = $this->getThisModel() )
        {
            // Push the model into the view (as default)
            $view->setModel( $model, true );
        }
        $profile_order_id = $session->get('profile_order_id',null,'j2store');
        $order     = F0FModel::getTmpInstance( 'Orders', 'J2StoreModel' )->initOrder($profile_order_id)->getOrder();
        $order->setBillingAddress($country_id,$zone_id,$postcode);
        $order->setShippingAddress($ship_country_id,$ship_zone_id,$ship_postcode);
        $items     = $order->getItems();

        $is_mobile = $session->get( 'is_mobile', '', 'j2store' );

        $mobile = ( $is_mobile ) ? '&mobile=mobile' : '';
        $link   = JRoute::_( 'index.php?option=com_j2store&view=carts' . $mobile );

        if ( count( $items ) < 1 )
        {
            $json['redirect'] = $link;
            return '';
        }

        //validate stock
        if ( $order->validate_order_stock() == false )
        {
            $json['redirect'] = $link;
            return '';
        }
        //prepare shipping
        // Checking whether shipping is required
        $showShipping = false;

        if ( J2Store::config()->get( 'show_shipping_address', 0 ) )
        {
            $showShipping = true;
        }

        if ( $isShippingEnabled = $order->isShippingEnabled() )
        {
            $showShipping = true;
        }
        $view->assign( 'showShipping', $showShipping );
        $user = JFactory::getUser();
        //prepare the billing address object
        //$this->prepare_billing_address( $view, $order, $user );

        $selectableBase = J2Store::getSelectableBase();
        $view->assign('fieldsClass', $selectableBase);

        $address = F0FTable::getAnInstance('address', 'J2StoreTable')->getClone();
        $view->assign('address',  $address );
        if($user->id) {
            //$address = $address_model->user_id($user->id)->getFirstItem();
            if(isset( $address->j2store_address_id ) && empty($address->j2store_address_id)){
                $userProfile = JUserHelper::getProfile( $user->id );
                $address->address_1 = isset($userProfile->profile['address1']) ? $userProfile->profile['address1']:'';
                $address->address_2 = isset($userProfile->profile['address2']) ? $userProfile->profile['address2']:'';
                $address->city = isset($userProfile->profile['city']) ? $userProfile->profile['city']:'';
                $address->zip = isset($userProfile->profile['postal_code']) ? $userProfile->profile['postal_code']:'';
                //
                $address->phone_1 = isset($userProfile->profile['phone']) ? $userProfile->profile['phone']:'';
                $address->phone_2 = isset($userProfile->profile['mobilephone']) ? $userProfile->profile['mobilephone']:'';
                $address->first_name = isset($userProfile->profile['first_name']) ? $userProfile->profile['first_name']:'';
                $address->last_name = isset($userProfile->profile['last_name']) ? $userProfile->profile['last_name']:'';
            }

        }
        // there could be three scenarios

        //1: Customer is already logged in
        /*if ( $user->id > 0 )
        {
            $this->customer_logged_in( $view, $order, $user );
        }else {
            $this->guest_checkout( $view, $order, $user );
        }*/

        if($showShipping){
            $shipping_layout = "shipping_yes";
            $shipping_method_form = $this->getShippingHtml( $shipping_layout, $order );
            
            $view->assign( 'showShipping', $showShipping );
            $view->assign( 'shipping_method_form', $shipping_method_form );
        }
        //custom fields
        $selectableBase = J2Store::getSelectableBase();
        $address_table = F0FTable::getAnInstance('Address', 'J2StoreTable');
        $payment_fields = $selectableBase->getFields('payment',$address_table,'address');

        $view->assign( 'payment_fields', $payment_fields );

        //process payment plugins
        $showPayment = true;
        if ((float)$order->order_total == (float)'0.00'  )
        {
            if(isset($order->show_payment_method) && $order->show_payment_method == 1){
                $showPayment = true;
            }else{
                $showPayment = false;
            }
            $app->triggerEvent("onJ2StoreChangeShowPaymentOnTotalZero", array( $order, &$showPayment ) );
        }
        $view->assign( 'showPayment', $showPayment );

        $payment_plugins = J2Store::plugin()->getPluginsWithEvent( 'onJ2StoreGetPaymentPlugins' );
        $default_method = J2Store::config()->get('default_payment_method', '');

        if(isset($data['payment_plugin']) && $data['payment_plugin']){
            $default_method = $data['payment_plugin'];
        }
        $plugins = array();
        if ($payment_plugins)
        {
            foreach ($payment_plugins as $plugin)
            {
                $results = $app->triggerEvent("onJ2StoreGetPaymentOptions", array( $plugin->element, $order ) );
                if (!in_array(false, $results, false))
                {
                    if(!empty($default_method) && $default_method == $plugin->element) {
                        $plugin->checked = true;
                        $html = $this->getPaymentForm( $plugin->element, true);
                        $view->assign( 'payment_form_div', $html);
                    }
                    $plugins[] = $plugin;
                }
            }
        }

        if (count($plugins) == 1)
        {
            $plugins[0]->checked = true;
            $html = $this->getPaymentForm( $plugins[0]->element, true);
            $view->assign( 'payment_form_div', $html);
        }

        $view->assign('plugins', $plugins);


        $view->assign( 'order', $order );
        // Set the layout
        $view->setLayout( 'validate' );
        //trigger on before checkout event
        J2Store::plugin()->event( 'BeforeCheckout', array( $order, &$view ) );
        // Display without caching
        ob_start ();
        $view->display ();
        $html = ob_get_contents ();
        ob_end_clean ();
        return $html;
    }

    public function saveBillingAndShipping($data,&$json){
        $app = JFactory::getApplication();
        $user = JFactory::getUser();
        $userHelper = J2Store::user();
        $session = JFactory::getSession();
        $address_model = F0FModel::getTmpInstance('Addresses', 'J2StoreModel');
        $store_address = $store = J2Store::storeProfile();

        if($user->id > 0){
            // save new address for existing user
            $selected_billing_address = isset($data['billing_address_type']) ? $data['billing_address_type'] : 'new';
            
            if (isset($selected_billing_address ) && $selected_billing_address == 'new') {
                $billing_address_id = $address_model->addAddress('billing',$data['new_billing_address']);
                //now get the address and save to session
                $address_info = $address_model->getItem($billing_address_id);
                $session->set('billing_address_id', $address_info->j2store_address_id, 'j2store');
            }elseif ($selected_billing_address == 'existing'){
                $address_id = isset($data['address_id']) ? $data['address_id'] : 0;
                $session->set('billing_address_id', $address_id, 'j2store');
            }
            $selected_shipping_address = isset($data['shipping_address_type']) ? $data['shipping_address_type'] : 'new';
            //$selected_shipping_address = $app->input->getString('shipping_address_type');
            if (isset($selected_shipping_address ) && $selected_shipping_address == 'new') {
                $shipping_address_id = $address_model->addAddress('shipping',$data['new_shipping_address']);
                //now get the address and save to session
                $ship_address_info = $address_model->getItem($shipping_address_id);
                $session->set('shipping_address_id', $ship_address_info->j2store_address_id, 'j2store');
            }elseif ($selected_shipping_address == 'existing'){
                $ship_address_id = isset($data['shipping_address_id']) ? $data['shipping_address_id'] : 0;
                $session->set('shipping_address_id', $ship_address_id, 'j2store');
            }


        }elseif(isset($data['billing']['create_account']) && $data['billing']['create_account']){
            // New User (Register)
            //now create the user
            $first_name = isset($data['billing']['first_name']) ? $data['billing']['first_name']: '';
            $last_name = isset($data['billing']['last_name']) ? $data['billing']['last_name']: '';
            $bill_email = isset($data['billing']['email']) ? $data['billing']['email']: '';
            $password = isset($data['billing']['password']) ? $data['billing']['password']: '';
            $confirm_password = isset($data['billing']['confirm']) ? $data['billing']['confirm']: '';
            // create the details array with new user info
            $details = array(
                'email' =>  $bill_email,
                'name' => $first_name.' '.$last_name,
                'username' =>  $bill_email,
                'password' => $password,
                'password2'=> $confirm_password
            );
            $msg = '';
            $user = $userHelper->createNewUser($details, $msg);
            $session->set('account', 'register', 'j2store');

            //now login the user
            if ( $userHelper->login(array('username' => $user->username, 'password' => $details['password']))) {
                $billing_address_id = $address_model->addAddress('billing',$data['billing']);

                //check if we have a country and zone id's. If not use the store address
                $country_id = isset($data['billing']['country_id']) ? $data['billing']['country_id']: '';
                if(empty($country_id)) {
                    $country_id = $store_address->get('country_id');
                }

                $zone_id = isset($data['billing']['zone_id']) ? $data['billing']['zone_id']: '';
                if(empty($zone_id)) {
                    $zone_id = $store_address->get('zone_id');
                }

                $postcode = isset($data['billing']['zip']) ? $data['billing']['zip']: '';
                if(empty($postcode)) {
                    $postcode = $store_address->get('zip');
                }

                $session->set('billing_address_id', $billing_address_id , 'j2store');
                $session->set('billing_country_id', $country_id, 'j2store');
                $session->set('billing_zone_id', $zone_id, 'j2store');
                $session->set('billing_postcode', $postcode, 'j2store');

                $shipping_address = isset($data['shipping_address']) ? $data['shipping_address']: 0;
                if ($shipping_address == 1) {
                    $session->set('shipping_address_id', $billing_address_id, 'j2store');
                    $session->set('shipping_country_id', $country_id, 'j2store');
                    $session->set('shipping_zone_id', $zone_id, 'j2store');
                    $session->set('shipping_postcode', $postcode, 'j2store');
                }else{

                    $shipping_address_id = $address_model->addAddress('shipping',$data['shipping']);
                    //check if we have a country and zone id's. If not use the store address
                    $shipping_country_id = isset($data['shipping']['country_id']) ? $data['shipping']['country_id']: '';
                    if(empty($shipping_country_id)) {
                        $shipping_country_id = $store_address->get('country_id');
                    }

                    $shipping_zone_id = isset($data['shipping']['zone_id']) ? $data['shipping']['zone_id']: '';
                    if(empty($shipping_zone_id)) {
                        $shipping_zone_id = $store_address->get('zone_id');
                    }

                    $shipping_postcode = isset($data['shipping']['zip']) ? $data['shipping']['zip']: '';
                    if(empty($shipping_postcode)) {
                        $shipping_postcode = $store_address->get('zip');
                    }
                    $session->set('shipping_address_id', $shipping_address_id, 'j2store');
                    $session->set('shipping_country_id', $shipping_country_id, 'j2store');
                    $session->set('shipping_zone_id', $shipping_zone_id, 'j2store');
                    $session->set('shipping_postcode', $shipping_postcode, 'j2store');
                }
                if(!$json) {
                    $json = J2Store::plugin()->eventWithArray('CheckoutAfterRegister');
                }

            }
        }elseif($user->id <= 0){

            // Guest
            if(isset($data['billing']) && $data['billing']){
                $billing_address_id = $address_model->addAddress('billing', $data['billing']);

                $session->set('billing_address_id', $billing_address_id,'j2store');
            }
            $guest_shipping = isset($data['shipping_address']) ? $data['shipping_address']: 0;

            if($guest_shipping == 0){
                $shipping_address_id = $address_model->addAddress('shipping', $data['shipping']);
                $session->set('shipping_address_id', $shipping_address_id,'j2store');
            }else{
                $session->set('shipping_address_id', $billing_address_id,'j2store');
            }
        }
    }

    public function validate_billing_address($data,&$json){
        $app = JFactory::getApplication();
        $session = JFactory::getSession();
        $address_model = F0FModel::getTmpInstance('Addresses', 'J2StoreModel');
        $store_address = $store = J2Store::storeProfile();
        $selectableBase = J2Store::getSelectableBase();

        J2Store::plugin()->event('BeforeCheckoutValidateBilling',array(&$json));
        //Has the customer selected an existing address?
        $selected_billing_address = $app->input->getString('billing_address_type');
        if (isset($selected_billing_address ) && $selected_billing_address == 'existing') {
            $selected_address_id =	$app->input->getInt('address_id');
            if (empty($selected_address_id)) {
                $json['error']['warning'] = JText::_('J2STORE_ADDRESS_SELECTION_ERROR');
            } elseif (!in_array($selected_address_id, array_keys($address_model->getAddresses('j2store_address_id')))) {
                $json['error']['warning'] = JText::_('J2STORE_ADDRESS_SELECTION_ERROR');
            }

            if (!$json) {
                $address_info = $address_model->getItem($selected_address_id);
                $session->set('billing_address_id', $selected_address_id, 'j2store');
                if ($address_info) {
                    //if country id is empty set it to the store country id
                    if(empty($address_info->country_id)) {
                        $session->set('billing_country_id',$store_address->get('country_id'), 'j2store');

                    } else {
                        $session->set('billing_country_id',$address_info->country_id, 'j2store');
                    }
                    $session->set('billing_zone_id',$address_info->zone_id, 'j2store');
                    $session->set('billing_postcode',$address_info->zip, 'j2store');
                }else {
                    $session->clear('billing_country_id', 'j2store');
                    $session->clear('billing_zone_id', 'j2store');
                    $session->clear('billing_postcode', 'j2store');
                }
                $session->clear('payment_method', 'j2store');
                $session->clear('payment_methods', 'j2store');
            }
        }elseif(isset($data['new_billing_address']) && !$json){
            $json = $selectableBase->validate($data['new_billing_address'], 'billing', 'address');
            if($json){
                $billing_field_error = $json['error'];
                unset($json['error']);
                foreach ( $billing_field_error as $json_key => $json_value){
                    $json['error']['new_billing_address'][$json_key] = $json_value;
                }
            }
            if(!$json) {
               // $address_id = $address_model->addAddress('billing',$data['billing']);
                //now get the address and save to session
               // $address_info = $address_model->getItem($address_id);

                //check if we have a country and zone id's. If not use the store address
                $country_id = isset($data['new_billing_address']['country_id']) ? $data['new_billing_address']['country_id']: '';
                if(empty($country_id)) {
                    $country_id = $store_address->get('country_id');
                }

                $zone_id = isset($data['new_billing_address']['zone_id']) ? $data['new_billing_address']['zone_id']: '';
                if(empty($zone_id)) {
                    $zone_id = $store_address->get('zone_id');
                }

                $postcode  = isset($data['new_billing_address']['zip']) ? $data['new_billing_address']['zip']: '';
                if(empty($postcode)) {
                    $postcode = $store_address->get('zip');
                }

               // $session->set('billing_address_id', $address_info->j2store_address_id, 'j2store');
                $session->set('billing_country_id', $country_id, 'j2store');
                $session->set('billing_zone_id',$zone_id, 'j2store');
                $session->set('billing_postcode',$postcode, 'j2store');
                $session->clear('payment_method', 'j2store');
                $session->clear('payment_methods', 'j2store');
            }
        }
    }

    public function validate_shipping_address($data,&$json){
        $app = JFactory::getApplication();
        $session = JFactory::getSession();
        $address_model = F0FModel::getTmpInstance('Addresses', 'J2StoreModel');
        $store_address = $store = J2Store::storeProfile();
        $selectableBase = J2Store::getSelectableBase();

        J2Store::plugin()->event('BeforeCheckoutValidateShipping',array(&$json));
        //Has the customer selected an existing address?
        $selected_shipping_address = $app->input->getString('shipping_address_type');
        if (isset($selected_shipping_address ) && $selected_shipping_address == 'existing') {
            $selected_address_id =	$app->input->getInt('shipping_address_id');
            if (empty($selected_address_id)) {
                $json['error']['warning'] = JText::_('J2STORE_ADDRESS_SELECTION_ERROR');
            } elseif (!in_array($selected_address_id, array_keys($address_model->getAddresses('j2store_address_id')))) {
                $json['error']['warning'] = JText::_('J2STORE_ADDRESS_SELECTION_ERROR');
            }

            if (!$json) {
                $address_info = $address_model->getItem($selected_address_id);
                $session->set('shipping_address_id', $selected_address_id, 'j2store');
                if ($address_info) {
                    //if country id is empty set it to the store country id
                    if(empty($address_info->country_id)) {
                        $session->set('shipping_country_id',$store_address->get('country_id'), 'j2store');

                    } else {
                        $session->set('shipping_country_id',$address_info->country_id, 'j2store');
                    }
                    $session->set('shipping_zone_id',$address_info->zone_id, 'j2store');
                    $session->set('shipping_postcode',$address_info->zip, 'j2store');
                }else {
                    $session->clear('shipping_country_id', 'j2store');
                    $session->clear('shipping_zone_id', 'j2store');
                    $session->clear('shipping_postcode', 'j2store');
                }
                $session->clear('payment_method', 'j2store');
                $session->clear('payment_methods', 'j2store');
            }
        }elseif(isset($data['new_shipping_address'])){
            $shiping_json = $selectableBase->validate($data['new_shipping_address'], 'shipping', 'address');
            if($shiping_json){
                $shipping_field_error = $shiping_json['error'];
                unset($shiping_json['error']);
                foreach ( $shipping_field_error as $json_key => $json_value){
                    $json['error']['new_shipping_address'][$json_key] = $json_value;
                }
            }
            if(!$json) {
                //$address_id = $address_model->addAddress('shipping',$data['new_shipping_address']);
                //now get the address and save to session
               // $address_info = $address_model->getItem($address_id);

                //check if we have a country and zone id's. If not use the store address
                $country_id = isset($data['new_shipping_address']['country_id']) ? $data['new_shipping_address']['country_id']: '';
                if(empty($country_id)) {
                    $country_id = $store_address->get('country_id');
                }

                $zone_id = isset($data['new_shipping_address']['zone_id']) ? $data['new_shipping_address']['zone_id']: '';
                if(empty($zone_id)) {
                    $zone_id = $store_address->get('zone_id');
                }

                $postcode  = isset($data['new_shipping_address']['zip']) ? $data['new_shipping_address']['zip']: '';
                if(empty($postcode)) {
                    $postcode = $store_address->get('zip');
                }

                //$session->set('shipping_address_id', $address_info->j2store_address_id, 'j2store');
                $session->set('shipping_country_id', $country_id, 'j2store');
                $session->set('shipping_zone_id',$zone_id, 'j2store');
                $session->set('shipping_postcode',$postcode, 'j2store');
                $session->clear('payment_method', 'j2store');
                $session->clear('payment_methods', 'j2store');
            }
        }
    }

    public function validate_register_billing_address($data,&$json){
        $app = JFactory::getApplication();
        $session = JFactory::getSession();
        $address_model = F0FModel::getTmpInstance('Addresses', 'J2StoreModel');
        $store_address = $store = J2Store::storeProfile();
        $selectableBase = J2Store::getSelectableBase();
        $redirect_url = JRoute::_('index.php?option=com_j2store&view=checkout');
        $userHelper = J2Store::user();
        //new Register User
        $billing_json = $selectableBase->validate($data['billing'], 'billing', 'address');
        if($billing_json){
            foreach ( $billing_json['error'] as $json_key => $json_value){
                $json['error']['billing'][$json_key] = $json_value;
            }
        }
        $shipping_address = isset($data['shipping_address']) ? $data['shipping_address']: 0;
        if($shipping_address == 0){
            $shipping_json = $selectableBase->validate($data['shipping'], 'shipping', 'address');
            if($shipping_json){
                foreach ( $shipping_json['error'] as $ship_json_key => $ship_json_value){
                    $json['error']['shipping'][$ship_json_key] = $ship_json_value;
                }
            }
        }
        if(!isset($data['payment'])){
            $data['payment'] = array();
        }
        $payment_json = $selectableBase->validate($data['payment'], 'payment', 'address');
        if($payment_json){
            foreach ( $payment_json['error'] as $payment_json_key => $payment_json_value){
                $json['error']['payment'][$payment_json_key] = $payment_json_value;
            }
        }


        $password = isset($data['billing']['password']) ? $data['billing']['password']: '';
        $confirm_password = isset($data['billing']['confirm']) ? $data['billing']['confirm']: '';
        $bill_email = isset($data['billing']['email']) ? $data['billing']['email']: '';

        //validate the password fields
        $userHelper->validatePassword($password,$confirm_password,$json);
        if(isset($json['error']['password'])){
            $json['error']['billing']['password'] = $json['error']['password'];
            unset($json['error']['password']);
        }
        if (isset($json['error']['confirm'])){
            $json['error']['billing']['confirm'] = $json['error']['confirm'];
            unset($json['error']['confirm']);
        }
        //check email
        if ((JString::strlen($bill_email) < 4)) {
            $json['error']['billing']['email'] = JText::_('J2STORE_EMAIL_REQUIRED');
        }

        //check email
        if($userHelper->emailExists($bill_email)){
            $json['error']['billing']['email'] = JText::_('J2STORE_EMAIL_EXISTS');
        }

        J2Store::plugin()->event('CheckoutValidateRegister', array(&$json));

        if(!$json){
            //check if we have a country and zone id's. If not use the store address
            $country_id = isset($data['billing']['country_id']) ? $data['billing']['country_id']: '';
            if(empty($country_id)) {
                $country_id = $store_address->get('country_id');
            }

            $zone_id = isset($data['billing']['zone_id']) ? $data['billing']['zone_id']: '';
            if(empty($zone_id)) {
                $zone_id = $store_address->get('zone_id');
            }

            $postcode = isset($data['billing']['zip']) ? $data['billing']['zip']: '';
            if(empty($postcode)) {
                $postcode = $store_address->get('zip');
            }

            $session->set('billing_country_id', $country_id, 'j2store');
            $session->set('billing_zone_id', $zone_id, 'j2store');
            $session->set('billing_postcode', $postcode, 'j2store');
            if ($shipping_address == 1) {
                $session->set('shipping_country_id', $country_id, 'j2store');
                $session->set('shipping_zone_id', $zone_id, 'j2store');
                $session->set('shipping_postcode', $postcode, 'j2store');
            }else{
                //check if we have a country and zone id's. If not use the store address
                $shipping_country_id = isset($data['shipping']['country_id']) ? $data['shipping']['country_id']: '';
                if(empty($shipping_country_id)) {
                    $shipping_country_id = $store_address->get('country_id');
                }

                $shipping_zone_id = isset($data['shipping']['zone_id']) ? $data['shipping']['zone_id']: '';
                if(empty($shipping_zone_id)) {
                    $shipping_zone_id = $store_address->get('zone_id');
                }

                $shipping_postcode = isset($data['shipping']['zip']) ? $data['shipping']['zip']: '';
                if(empty($shipping_postcode)) {
                    $shipping_postcode = $store_address->get('zip');
                }
                $session->set('shipping_country_id', $shipping_country_id, 'j2store');
                $session->set('shipping_zone_id', $shipping_zone_id, 'j2store');
                $session->set('shipping_postcode', $shipping_postcode, 'j2store');
            }
        }
        if(!$json) {
            $json = J2Store::plugin()->eventWithArray('CheckoutAfterRegister');
        }
        /*if(!$json){
            //now create the user
            $first_name = isset($data['billing']['first_name']) ? $data['billing']['first_name']: '';
            $last_name = isset($data['billing']['last_name']) ? $data['billing']['last_name']: '';
            // create the details array with new user info
            $details = array(
                'email' =>  $bill_email,
                'name' => $first_name.' '.$last_name,
                'username' =>  $bill_email,
                'password' => $password,
                'password2'=> $confirm_password
            );
            $msg = '';
            $user = $userHelper->createNewUser($details, $msg);
            $session->set('account', 'register', 'j2store');

            //now login the user
            if ( $userHelper->login(array('username' => $user->username, 'password' => $details['password']))) {
                $billing_address_id = $address_model->addAddress('billing',$data['billing']);

                //check if we have a country and zone id's. If not use the store address
                $country_id = isset($data['billing']['country_id']) ? $data['billing']['country_id']: '';
                if(empty($country_id)) {
                    $country_id = $store_address->get('country_id');
                }

                $zone_id = isset($data['billing']['zone_id']) ? $data['billing']['zone_id']: '';
                if(empty($zone_id)) {
                    $zone_id = $store_address->get('zone_id');
                }

                $postcode = isset($data['billing']['zip']) ? $data['billing']['zip']: '';
                if(empty($postcode)) {
                    $postcode = $store_address->get('zip');
                }

                $session->set('billing_address_id', $billing_address_id , 'j2store');
                $session->set('billing_country_id', $country_id, 'j2store');
                $session->set('billing_zone_id', $zone_id, 'j2store');
                $session->set('billing_postcode', $postcode, 'j2store');


                if ($shipping_address == 1) {
                    $session->set('shipping_address_id', $billing_address_id, 'j2store');
                    $session->set('shipping_country_id', $country_id, 'j2store');
                    $session->set('shipping_zone_id', $zone_id, 'j2store');
                    $session->set('shipping_postcode', $postcode, 'j2store');
                }else{

                    $shipping_address_id = $address_model->addAddress('shipping',$data['shipping']);
                    //check if we have a country and zone id's. If not use the store address
                    $shipping_country_id = isset($data['shipping']['country_id']) ? $data['shipping']['country_id']: '';
                    if(empty($shipping_country_id)) {
                        $shipping_country_id = $store_address->get('country_id');
                    }

                    $shipping_zone_id = isset($data['shipping']['zone_id']) ? $data['shipping']['zone_id']: '';
                    if(empty($shipping_zone_id)) {
                        $shipping_zone_id = $store_address->get('zone_id');
                    }

                    $shipping_postcode = isset($data['shipping']['zip']) ? $data['shipping']['zip']: '';
                    if(empty($shipping_postcode)) {
                        $shipping_postcode = $store_address->get('zip');
                    }
                    $session->set('shipping_address_id', $shipping_address_id, 'j2store');
                    $session->set('shipping_country_id', $shipping_country_id, 'j2store');
                    $session->set('shipping_zone_id', $shipping_zone_id, 'j2store');
                    $session->set('shipping_postcode', $shipping_postcode, 'j2store');
                }
                if(!$json) {
                    $json = J2Store::plugin()->eventWithArray('CheckoutAfterRegister');
                }

            }
        }*/
        $session->clear('guest', 'j2store');
        $session->clear('shipping_method', 'j2store');
        $session->clear('shipping_methods', 'j2store');
        $session->clear('payment_method', 'j2store');
        $session->clear('payment_methods', 'j2store');
    }

    public function validate_guest_address($data,&$json){
        //guest User
        $app = JFactory::getApplication();
        $session = JFactory::getSession();
        $address_model = F0FModel::getTmpInstance('Addresses', 'J2StoreModel');
        $store_address = $store = J2Store::storeProfile();
        $selectableBase = J2Store::getSelectableBase();
        $redirect_url = JRoute::_('index.php?option=com_j2store&view=checkout');
        $guest = $session->get('guest', array(), 'j2store');
        $params = J2Store::config();
        // Validate if customer is logged in.
        if (JFactory::getUser()->id) {
            $json['redirect'] = $redirect_url;
            $json['error']['warning'] = JText::_("J2STORE_USER_LOGED_IN_PLEASE_REDIRECT");
        }

        // Validate order
        $order = F0FModel::getTmpInstance('Orders', 'J2StoreModel')->initOrder()->getOrder();
        if(count($order->getItems()) < 1) {
            $json['redirect'] = $redirect_url;
        }

        // Check if guest checkout is avaliable.
        //TODO prevent if products have downloads also
        /*if (!$params->get('allow_guest_checkout')) {
            $json['redirect']['allow_checkout'] = $redirect_url;
        }*/

        if (!$json && isset($data['billing'])) {

            $selectableBase =J2Store::getSelectableBase();
            $billing_json = $selectableBase->validate($data['billing'], 'billing', 'address');
            
            if($billing_json){
                foreach ( $billing_json['error'] as $json_key => $json_value){
                    $json['error']['billing'][$json_key] = $json_value;
                }
            }
            $bill_email = isset($data['billing']['email']) ? $data['billing']['email']: '';
            //check email
            if ((JString::strlen($bill_email) < 4)) {
                $json['error']['billing']['email'] = JText::_('J2STORE_EMAIL_REQUIRED');
            }
            J2Store::plugin()->event('CheckoutValidateGuest',array(&$json,&$data));
        }elseif(!$json){
            $json['error']['warning'] = JText::_('J2STORE_BILLING_ADDRESS_NOT_FOUND');
        }


        if (!$json && isset($data['billing'])) {
            //now assign the post data to the guest billing array.
            /*foreach($data as $key=>$value) {
                $guest['billing'][$key] = $value;
            }*/

            $guest['billing'] = $data['billing'];
            //check if we have a country and zone id's. If not use the store address
            $country_id = isset($data['billing']['country_id']) ? $data['billing']['country_id']: '';
            if(empty($country_id)) {
                $country_id = $store_address->get('country_id');
            }

            $zone_id = isset($data['billing']['zone_id']) ? $data['billing']['zone_id']: '';
            if(empty($zone_id)) {
                $zone_id = $store_address->get('zone_id');
            }

            $postcode = isset($data['billing']['zip']) ? $data['billing']['zip']: '';
            if(empty($postcode)) {
                $postcode = $store_address->get('zip');
            }
            ///returns an object
            $country_info = F0FModel::getTmpInstance('Countries', 'J2StoreModel')->getItem($country_id);

            //save to address table before you proceed.
            //$address_model->addAddress('billing', $guest['billing']);

            if ($country_info) {
                $guest['billing']['country_name'] = $country_info->country_name;
                $guest['billing']['iso_code_2'] = $country_info->country_isocode_2;
                $guest['billing']['iso_code_3'] = $country_info->country_isocode_3;
            } else {
                $guest['billing']['country_name'] = '';
                $guest['billing']['iso_code_2'] = '';
                $guest['billing']['iso_code_3'] = '';
            }

            $zone_info = F0FModel::getTmpInstance('Zones', 'J2StoreModel')->getItem($zone_id);

            if ($zone_info) {
                $guest['billing']['zone_name'] = $zone_info->zone_name;
                $guest['billing']['zone_code'] = $zone_info->zone_code;
            } else {
                $guest['billing']['zone_name'] = '';
                $guest['billing']['zone_code'] = '';
            }
            $guest_shipping = isset($data['shipping_address']) ? $data['shipping_address']: 0;
            if (!empty($guest_shipping)) {
                $guest['shipping_address'] = true;
            } else {
                $guest['shipping_address'] = false;
            }

            // Default billing address
            $session->set('billing_country_id', $country_id, 'j2store');
            $session->set('billing_zone_id', $zone_id, 'j2store');
            $session->set('billing_postcode', $postcode, 'j2store');
        }
        if (isset($data['shipping_address']) && $data['shipping_address'] == 0) {
            if (isset($data['shipping'])) {

                $selectableBase =J2Store::getSelectableBase();
                $shipping_json = $selectableBase->validate($data['shipping'], 'shipping', 'address');
                if($shipping_json){
                    foreach ( $shipping_json['error'] as $shipping_json_key => $shipping_json_value){
                        $json['error']['shipping'][$shipping_json_key] = $shipping_json_value;
                    }
                }
                J2Store::plugin()->event('CheckoutValidateGuestShipping',array(&$json,&$data));
            }elseif(!$json){
                $json['error']['warning'] = JText::_('J2STORE_SHIPPING_ADDRESS_NOT_FOUND');
            }
            /*foreach($data as $key=>$value) {
                $guest['shipping'][$key] = $value;
            }*/
            if (!$json && isset($data['shipping'])) {
                //check if we have a country and zone id's. If not use the store address
                $shipping_country_id = isset($data['shipping']['country_id']) ? $data['shipping']['country_id']: '';
                if(empty($shipping_country_id)) {
                    $shipping_country_id = $store_address->get('country_id');
                }

                $shipping_zone_id = isset($data['shipping']['zone_id']) ? $data['shipping']['zone_id']: '';
                if(empty($shipping_zone_id)) {
                    $shipping_zone_id = $store_address->get('zone_id');
                }

                $shipping_postcode = isset($data['shipping']['zip']) ? $data['shipping']['zip']: '';
                if(empty($shipping_postcode)) {
                    $shipping_postcode = $store_address->get('zip');
                }
                ///returns an object
                $shipping_country_info = F0FModel::getTmpInstance('Countries', 'J2StoreModel')->getItem($shipping_country_id);

                $guest['shipping'] = $data['shipping'];
                $guest['shipping']['email'] = isset($data['billing']['email']) ? $data['billing']['email']: '';
                //save to address table before you proceed.
                //  $address_model->addAddress('shipping', $guest['shipping']);

                if ($shipping_country_info) {
                    $guest['shipping']['country_name'] = $shipping_country_info->country_name;
                    $guest['shipping']['iso_code_2'] = $shipping_country_info->country_isocode_2;
                    $guest['shipping']['iso_code_3'] = $shipping_country_info->country_isocode_3;
                } else {
                    $guest['shipping']['country_name'] = '';
                    $guest['shipping']['iso_code_2'] = '';
                    $guest['shipping']['iso_code_3'] = '';
                }
                $shipping_zone_info = F0FModel::getTmpInstance('Zones', 'J2StoreModel')->getItem($shipping_zone_id);
                if ($shipping_zone_info) {
                    $guest['shipping']['zone_name'] = $shipping_zone_info->zone_name;
                    $guest['shipping']['zone_code'] = $shipping_zone_info->zone_code;
                } else {
                    $guest['shipping']['zone_name'] = '';
                    $guest['shipping']['zone_code'] = '';
                }
                // Default Shipping Address
                $session->set('shipping_country_id', $shipping_country_id, 'j2store');
                $session->set('shipping_zone_id', $shipping_zone_id, 'j2store');
                $session->set('shipping_postcode', $shipping_postcode, 'j2store');
            }
        }else{
            $guest['shipping'] = isset($guest['billing']) ? $guest['billing']: array();
        }

        //now set the guest values to the session
        $session->set('guest', $guest, 'j2store');
        $session->set('account', 'guest', 'j2store');

        //first validate custom fields
        if(isset($data['payment'])){
            $selectableBase = J2Store::getSelectableBase();
            $payment_json = $selectableBase->validate($data['payment'], 'payment', 'address');
            if($payment_json){
                foreach ( $payment_json['error'] as $payment_json_key => $payment_json_value){
                    $json['error']['payment'][$payment_json_key] = $payment_json_value;
                }
            }
        }

        $session->clear('shipping_method', 'j2store');
        $session->clear('shipping_methods', 'j2store');
        $session->clear('payment_method', 'j2store');
        $session->clear('payment_methods', 'j2store');
    }

    public function validateSelectShipping($values,$order){
        $error = 0;

        if (isset ( $values ['shippingrequired'] )) {
            if ($values ['shippingrequired'] == 1 && empty ( $values ['shipping_plugin'] )) {
                throw new Exception ( JText::_ ( 'J2STORE_CHECKOUT_SELECT_A_SHIPPING_METHOD' ) );
                return false;
            }
        }

        if (( float ) $order->order_total == ( float ) '0.00') {
            return true;
        }

        // trigger the plugin's validation function
        // no matter what, fire this validation plugin event for plugins that extend the checkout workflow
        $results = array ();
        $results = JFactory::getApplication ()->triggerEvent ( "onValidateSelectShipping", array (
            $values
        ) );

        for($i = 0; $i < count ( $results ); $i ++) {
            $result = $results [$i];
            if (! empty ( $result->error )) {
                throw new Exception ( $result->message );
                return false;
            }
        }

        if ($error == '1') {
            return false;
        }

        return true;
    }

    public function validateSelectPayment($payment_plugin, $values){
        $response = array ();
        $response ['msg'] = '';
        $response ['error'] = '';

        $app = JFactory::getApplication ();
        JPluginHelper::importPlugin ( 'j2store' );
        // verify the form data
        $results = array ();
        $results = $app->triggerEvent ( "onJ2StoreGetPaymentFormVerify", array (
            $payment_plugin,
            $values
        ) );

        for($i = 0; $i < count ( $results ); $i ++) {
            $result = $results [$i];
            if (! empty ( $result->error )) {
                $response ['msg'] = $result->message;
                $response ['error'] = '1';
            }
        }
        if ($response ['error']) {
            throw new Exception ( $response ['msg'] );
            return false;
        } else {
            return true;
        }
        return false;
    }


    function confirm() {

        //no cache
        J2Store::utilities()->nocache();

        $app = JFactory::getApplication();
        $session = JFactory::getSession();
        $user = JFactory::getUser();
        $params = J2Store::config();
        $address_model = F0FModel::getTmpInstance('Addresses', 'J2StoreModel');
        JPluginHelper::importPlugin ('j2store');

        $errors = array();

        $view = $this->getThisView();
        if ($model = $this->getThisModel())
        {
            // Push the model into the view (as default)
            $view->setModel($model, true);
        }

        //get the payment plugin form values set in the session.
        if($session->has('payment_values', 'j2store')) {
            $values = $session->get('payment_values', array(), 'j2store');
            //backward compatibility. TODO: change the way the plugin gets its data
            foreach($values as $name=>$value) {
                $app->input->set($name, $value);
            }
        }
        $session->set ( 'force_calculate_shipping', 1, 'j2store' );
        $shipping_address_id = $session->get('shipping_address_id',0,'j2store');
        $billing_address_id = $session->get('billing_address_id',0,'j2store');
        if($shipping_address_id){
            $address_table = F0FTable::getAnInstance('Address', 'J2StoreTable')->getClone();
            $address_table->load($shipping_address_id);
            // shipping session
            $session->set ( 'shipping_country_id', $address_table->country_id,'j2store');
            $session->set ( 'shipping_zone_id', $address_table->zone_id,'j2store');
            $session->set ( 'shipping_postcode', $address_table->zip,'j2store');
        }
        if($billing_address_id){
            $bill_address_table = F0FTable::getAnInstance('Address', 'J2StoreTable')->getClone();
            $bill_address_table->load($shipping_address_id);
            // billing session
            $session->set ( 'billing_country_id', $bill_address_table->country_id,'j2store');
            $session->set ( 'billing_zone_id', $bill_address_table->zone_id,'j2store');
            $session->set ( 'billing_postcode', $bill_address_table->zip,'j2store');
        }


        //validate the order
        try {
            $orders_model = F0FModel::getTmpInstance('Orders', 'J2StoreModel');

            //if we already have the order in the session, then it might be an update. So pass it
            $order_id = $app->getUserState( 'j2store.order_id', null);
            $order = $orders_model->initOrder($order_id)->getOrder();
            $orders_model->validateOrder($order);
            //plugin trigger
            $app->triggerEvent( "onJ2StoreAfterOrderValidate", array(&$order) );
        }catch (Exception $e) {
            $errors[]= $e->getMessage();
        }

        //on before checkout for payment script
        J2Store::plugin()->event( 'BeforeCheckout', array( $order, &$view ) );

        //Extra watch fix
        if(!$session->has('payment_method', 'j2store')) {
            $payment_values = $session->get('payment_values', array(), 'j2store');
            $payment_method = isset($payment_values['payment_plugin']) ? $payment_values['payment_plugin'] : '';
            $session->set('payment_method', $payment_method, 'j2store');
        }
        $orderpayment_type = $session->get('payment_method', '', 'j2store');
        //showPayment
        $showPayment = true;
        if ((float)$order->order_total == (float)'0.00')
        {
            $showPayment = false;
            $orderpayment_type = 'free';
            $app->triggerEvent("onJ2StoreChangeShowPaymentOnTotalZero", array( $order, &$showPayment ) );
            if($showPayment === true){
                $orderpayment_type = $session->get('payment_method', '', 'j2store');
            } else {
                // in the case of orders with a value of 0.00, we redirect to the confirmPayment page
                $free_redirect = JRoute::_( 'index.php?option=com_j2store&view=checkout&task=confirmPayment' );
                $view->assign('free_redirect', $free_redirect);
            }
        }
        $view->assign( 'showPayment', $showPayment );


        // Validate if payment method has been set.
        $orderpayment_type = trim($orderpayment_type);
        if (($showPayment == true && !$session->has('payment_method', 'j2store')) || empty($orderpayment_type)) {
            $errors[] = JText::_('J2STORE_CHECKOUT_ERROR_PAYMENT_METHOD_NOT_SELECTED');
        }

        if(!$errors) {
            //$orderpayment_type = $session->get('payment_method', '', 'j2store');

            //trigger onJ2StoreBeforePayment event
            if ($showPayment == true && !empty($orderpayment_type)) {
                //Since 3.2, this is not required. Fees API is implemented.
                //	$results = $app->triggerEvent( "onJ2StoreBeforePayment", array($orderpayment_type, $order) );
            }

            // in the case of orders with a value of 0.00, use custom values
//			if ( (float) $order->order_total == (float)'0.00' )
//			{
//				$orderpayment_type = 'free';
//				$transaction_status = JText::_( "J2STORE_COMPLETE" );
//			}

            $order->orderpayment_type = $orderpayment_type;

            try {
                $order = $order->saveOrder();
                // IMPORTANT: Store the order_id in the user's session for the postPayment "View Invoice" link
                $view->assign('order', $order);

                $app->setUserState( 'j2store.order_id', $order->order_id );
                $app->setUserState( 'j2store.orderpayment_id', $order->j2store_order_id );
                $app->setUserState( 'j2store.order_token', $order->token);
                // in the case of orders with a value of 0.00, we redirect to the confirmPayment page
//				if ( (float) $order->order_total == (float)'0.00' )
//				{
//					$free_redirect = JRoute::_( 'index.php?option=com_j2store&view=checkout&task=confirmPayment' );
//					$view->assign('free_redirect', $free_redirect);
//				}

                $values = array();
                $values['order_id'] = $order->order_id;
                $values['orderpayment_id'] = $order->j2store_order_id;
                $values['orderpayment_amount'] = $order->order_total;
                $values['order'] = $order;

                $results = $app->triggerEvent( "onJ2StorePrePayment", array( $orderpayment_type, $values));

                // Display whatever comes back from Payment Plugin for the onPrePayment
                $html = "";
                for ($i=0; $i<count($results); $i++)
                {
                    $html .= $results[$i];
                }

                $view->assign('plugin_html', $html);

            } catch (Exception $e) {
                $errors[] = $e->getMessage();
            }

        }

        if(count($errors)) {
            $view->assign('error', implode('/n', $errors));
        }

        // Set display
        $view->setLayout('default_confirm');


        $html = '';

        ob_start();
        $view->display();
        $html .= ob_get_contents();
        ob_end_clean();
        echo $html;
    }

	//utility functions //

	public function getAllAddresses() {
		$user = JFactory::getUser();
		//get all address
		if($user->id) {
			$address_model = F0FModel::getTmpInstance('Addresses', 'J2StoreModel');
			$addresses = $address_model->user_id($user->id)->getList();
		}else {
			$addresses = array();
		}
		return $addresses;
	}


	public function saveAddress(){
        $app = JFactory::getApplication();
        $data = $app->input->getArray($_REQUEST);
        $address_model = F0FModel::getTmpInstance('Addresses', 'J2StoreModel');
        $store_address = $store = J2Store::storeProfile();
        $selectableBase = J2Store::getSelectableBase();
        $session = JFactory::getSession();
        $json = array();

        if(isset($data['address_type']) && !empty($data['address_type'])){
            foreach ($data['address_data'] as $s_data){
                if($data['address_type'] == 'billing'){
                    $data['billing'][$s_data['name']] = $s_data['value'];
                }elseif($data['address_type'] == 'shipping'){
                    $data['shipping'][substr($s_data['name'],9)] = $s_data['value'];
                }

            }
            if($data['address_type'] == 'billing'){
                J2Store::plugin()->event('BeforeCheckoutValidateBilling',array(&$json));
                $bill_json = $selectableBase->validate($data['billing'], 'billing', 'address');
                if($bill_json){
                    $billing_field_error = $bill_json['error'];
                    unset($bill_json['error']);
                    foreach ( $billing_field_error as $json_key => $json_value){
                        $json['error']['billing'][$json_key] = $json_value;
                    }
                }
                J2Store::plugin()->event('CheckoutValidateBilling',array(&$json));
                if(!$json){
                    $address_id = $address_model->addAddress('billing',$data['billing']);
                    //now get the address and save to session
                    // $address_info = $address_model->getItem($address_id);

                    //check if we have a country and zone id's. If not use the store address
                    $country_id = isset($data['billing']['country_id']) ? $data['billing']['country_id']: '';
                    if(empty($country_id)) {
                        $country_id = $store_address->get('country_id');
                    }

                    $zone_id = isset($data['billing']['zone_id']) ? $data['billing']['zone_id']: '';
                    if(empty($zone_id)) {
                        $zone_id = $store_address->get('zone_id');
                    }

                    $postcode  = isset($data['billing']['zip']) ? $data['billing']['zip']: '';
                    if(empty($postcode)) {
                        $postcode = $store_address->get('zip');
                    }

                    $session->set('billing_address_id', $address_id, 'j2store');
                    $session->set('billing_country_id', $country_id, 'j2store');
                    $session->set('billing_zone_id',$zone_id, 'j2store');
                    $session->set('billing_postcode',$postcode, 'j2store');
                }
            }elseif($data['address_type'] == 'shipping'){

                J2Store::plugin()->event('BeforeCheckoutValidateShipping',array(&$json));
                $ship_json = $selectableBase->validate($data['shipping'], 'shipping', 'address');
                if($ship_json){
                    $ship_field_error = $ship_json['error'];
                    unset($ship_json['error']);
                    foreach ( $ship_field_error as $json_key => $json_value){
                        $json['error']['shipping'][$json_key] = $json_value;
                    }
                }
                J2Store::plugin()->event('CheckoutValidateShipping',array(&$json));
                if(!$json){
                    $address_id = $address_model->addAddress('shipping',$data['shipping']);
                    //now get the address and save to session
                    // $address_info = $address_model->getItem($address_id);

                    //check if we have a country and zone id's. If not use the store address
                    $country_id = isset($data['shipping']['country_id']) ? $data['shipping']['country_id']: '';
                    if(empty($country_id)) {
                        $country_id = $store_address->get('country_id');
                    }

                    $zone_id = isset($data['shipping']['zone_id']) ? $data['shipping']['zone_id']: '';
                    if(empty($zone_id)) {
                        $zone_id = $store_address->get('zone_id');
                    }

                    $postcode  = isset($data['shipping']['zip']) ? $data['shipping']['zip']: '';
                    if(empty($postcode)) {
                        $postcode = $store_address->get('zip');
                    }

                    $session->set('shipping_address_id', $address_id, 'j2store');
                    $session->set('shipping_country_id', $country_id, 'j2store');
                    $session->set('shipping_zone_id',$zone_id, 'j2store');
                    $session->set('shipping_postcode',$postcode, 'j2store');
                }
            }
            if(!$json){
                $json['success'] = 1;
                $json['redirect'] = JRoute::_('index.php?option=com_easycheckout&view=checkouts');
            }
        }else{
            $json['error'] = JText::_('J2STORE_ADDRESS_TYPE_NOT_FOUND');
        }
        echo json_encode($json);
        $app->close();
    }



}
